@@ -15,7 +15,7 @@
   }
   else
   {
-    stop("Bootstrap not implemented for this class.\n")
+    stop("Bootstrap confidence intervals not implemented for this class.\n")
   }
   obs <- MODEL$obs
   nvar<- MODEL$nvar
@@ -30,7 +30,7 @@
   resorig <- scale(MODEL$residuals, scale = FALSE)
   residup <-resorig * -0.6180339887498949025257 # (1+sqrt(5))*0.5
   residdown <- resorig * 1.618033988749894902526 # (1+sqrt(5))*0.5
-  fitted<-MODEL$fitted
+  fitted <- MODEL$fitted
   for (i in 1:runs) {
     if (tboot == "wild"){
       index <- ifelse(apply(resorig, 2, stats::rbinom, size=1, prob=0.7236067977499789360962) == 1, TRUE, FALSE)
@@ -42,10 +42,7 @@
       ystar <- fitted + resid
     }
     MODEL$y <- ystar
-    if(any(class(x) == "tvsure"))
-      modelboot <- update(MODEL, y = ystar)
-    else
-      modelboot <- update(MODEL)
+    modelboot <- update(MODEL)
     BOOT[[i]] <- modelboot$tvcoef
   }
   return(BOOT)
@@ -74,42 +71,43 @@
   }
   p <- VAR$p
   neq <- VAR$neq
-  X <- VAR$datamat[, -c(1:neq)]
+  X <- VAR$x
   obs <- VAR$obs
   total <- VAR$totobs
   type <- VAR$type
-  params <- ncol(x$datamat[, -c(1:neq)])
+  params <- NCOL(x$x)
   B <- tvBcoef(VAR)
   BOOT <- vector("list", runs)
   ystar <- matrix(0, nrow = total, ncol = neq)
-  colnames(ystar) <- colnames(VAR$y)
-  y.names <- colnames(ystar)
+  y.names <- colnames(VAR$y)
+  colnames(ystar) <- y.names
   Zdet <- NULL
-  if (ncol(X) > (neq * (p + 1)))
+  if (NCOL(X) > (neq * (p + 1)))
   {#in case there are exogen variables
-    Zdet <- as.matrix(X[, (neq * (p + 1) + 1):ncol(X)])
+    Zdet <- as.matrix(X[, (neq * (p + 1) + 1):NCOL(X)])
   }
   resorig <- scale(VAR$residuals, scale = FALSE)
-  Sigmaorig<-crossprod(resorig)/(obs - params)
-  eSigma<-eigen(Sigmaorig, TRUE)
-  Sigma<-diag(eSigma$values^0.5)%*%eSigma$vector
+  Sigmaorig <- crossprod(resorig)/(obs - params)
+  eSigma <- eigen(Sigmaorig, TRUE)
+  Sigma <- diag(eSigma$values^0.5)%*%eSigma$vector
   resid <-resorig
-  fitted<-VAR$fitted
+  fitted <- VAR$fitted
+  yorig <- VAR$y.orig
   for (i in 1:runs)
   {
     if (tboot == "wild")
     {
-      index <-ifelse(apply(resorig, 2, stats::rbinom, size=1, prob=0.7236067977499789360962) == 1, TRUE, FALSE)
-      resid  <-resorig*-0.6180339887498949025257 # (1-sqrt(5))*0.5
-      resid [!index]<-resorig[!index]*1.618033988749894902526 # (1+sqrt(5))*0.5
-      lasty <- c(t(VAR$y[p:1, ]))
-      ystar[c(1:p), ] <- VAR$y[c(1:p), ]
+      index <- ifelse(apply(resorig, 2, stats::rbinom, size=1, prob=0.7236067977499789360962) == 1, TRUE, FALSE)
+      resid <- resorig*-0.6180339887498949025257 # (1-sqrt(5))*0.5
+      resid [!index] <- resorig[!index]*1.618033988749894902526 # (1+sqrt(5))*0.5
+      lasty <- c(t(yorig[p:1, ]))
+      ystar[c(1:p), ] <- yorig[c(1:p), ]
     }
     else if (tboot == "wild2")
     {
       resid  <-resorig*stats::rnorm(obs*neq)
-      lasty <- c(t(VAR$y[p:1, ]))
-      ystar[c(1:p), ] <- VAR$y[c(1:p), ]
+      lasty <- c(t(yorig[p:1, ]))
+      ystar[c(1:p), ] <- yorig[c(1:p), ]
     }
     for (j in 1:obs)
     {
@@ -120,8 +118,15 @@
       ystar[j + p, ] <- B[j,,]%*%Z + resid[j, ]
       lasty <- c(ystar[j + p, ], lasty)
     }
-    varboot <- update(VAR, y = ystar)
-    BOOT[[i]] <- .tvIRF( x = varboot, impulse = impulse, response = response, y.names = y.names,
+    VAR$y.orig <- ystar
+    VAR$y <- ystar[-c(1:p),]
+    VAR$resid <- resid
+    temp <- stats::embed(ystar, dimension = p + 1)[, -c(1:neq)]
+    if(VAR$type == "const")
+      temp <- cbind(temp, rep(1, obs))
+    VAR$x[, 1:NCOL(temp)] <- temp
+    varboot <- update(VAR)
+    BOOT[[i]] <- .tvIRF(x = varboot, impulse = impulse, response = response, y.names = y.names,
                          n.ahead = n.ahead, ortho = ortho, cumulative = cumulative,
                          ortho.cov = ortho.cov, bw.cov = bw.cov)$irf
   }
