@@ -1,15 +1,27 @@
 #' Time-Varying Ordinary Least Squares
 #'
-#' \code{tvOLS} is used to fit univariate linear models with time-varying coefficients.
+#' \code{tvOLS} estimate time-varying coefficient of univariate 
+#' linear models using the kernel smoothing OLS.
 #'
+#' @param x an object used to select a method.
+#' @param ... Other arguments passed to specific methods.
+#' @return \code{tvGLS} returns a list containing:
+#' \item{tvcoef}{A vector of length obs, number of observations with
+#' the time-varying estimates.}
+#' \item{fitted}{A vector of length obs with the fited values from the estimation.}
+#' \item{residuals}{A vector of length obs with the residuals from the estimation.}
+#' @export
+#' @import methods
+tvOLS <- function(x, ...) UseMethod("tvOLS", x)
+
 #' @param x A matrix with all regressors.
 #' @param y A vector with dependent variable.
 #' @param z A vector with the variable over which coefficients are smooth over.
+#' @param ez (optional) A scalar or vector with the smoothing values. If 
+#' values are included then the vector z is used.
 #' @param bw A numeric vector.
 #' @inheritParams tvSURE
 #' @param singular.ok	Logical. If FALSE, a singular model is an error.
-#' @return A list with the estimates, fitted and residuals values.
-#'
 #' @examples
 #' tau <- seq(1:500)/500
 #' beta <- data.frame(beta1 = sin(2*pi*tau), beta2= 2*tau)
@@ -24,29 +36,30 @@
 #' lines(tau, coef.tvlm[, 1], col = 4)
 #' legend("topright", c(expression(beta[1]), "lm", "tvlm"), col = c(1, 2, 4), bty="n", lty = 1)
 #'
-#' @seealso \code{\link{bw}} for bandwidth selection and \code{\link{tvLM}}
+#' @seealso \code{\link{bw}} for bandwidth selection, \code{\link{tvLM}} and
+#' \code{\link{tvAR}}.
+#' @method tvOLS matrix
 #' @export
 
-tvOLS <- function(x, y, z = NULL, bw, est = c("lc", "ll"), 
-                  tkernel = c("Epa", "Gaussian"), singular.ok = singular.ok)
+tvOLS.matrix <- function(x, y, z = NULL, ez = NULL, bw, est = c("lc", "ll"), 
+                  tkernel = c("Epa", "Gaussian"), 
+                  singular.ok = singular.ok, ...)
 {
   x <- as.matrix(x)
   y <- as.numeric(y)
+  is.predict <- ifelse (is.null(ez), FALSE, TRUE)
   obs <- NROW(x)
   if(!identical(length(y), obs))
     stop("\nDimensions of 'x' and 'y' are not compatible.\n")
   if(!is.numeric(bw))
-    stop ("Parameter 'bw' should be a scalar. \n")
+    stop ("Argument 'bw' should be a scalar. \n")
   tkernel <- match.arg(tkernel)
   est <- match.arg(est)
   if(!(tkernel %in% c("Epa", "Gaussian")))
     tkernel <- "Epa"
   if(!(est %in% c("lc", "ll")))
     est <- "lc"
-  fitted <- numeric(obs)
-  resid <- numeric(obs)
   nvar <- NCOL(x)
-  theta <- matrix(0, obs, nvar)
   if(!is.null(z))
   {
     if(length(z) != obs)
@@ -55,13 +68,19 @@ tvOLS <- function(x, y, z = NULL, bw, est = c("lc", "ll"),
   }
   else
     grid <- (1:obs)/obs
-  for (t in 1:obs)
+  if (is.null(ez))
+    ez <- grid
+  eobs <- NROW(ez)
+  fitted <- numeric(eobs)
+  resid <- numeric(eobs)
+  theta <- matrix(0, eobs, nvar)
+  for (t in 1:eobs)
   { 
-    x2 <- grid - grid[t]
+    x2 <- grid - ez[t]
     kernel.bw <- .kernel(x = x2, bw = bw, tkernel = tkernel, N = 1)
     myindex <- which(kernel.bw != 0)
     if (length (myindex) < 3)
-      stop("\nBandwidth is too small.\n.")
+      stop("Bandwidth is too small or 'ez' is too large.\n")
     if (sum(myindex) == 0)
       return (.Machine$double.xmax)
     xtemp <- x[myindex, ]
@@ -72,10 +91,67 @@ tvOLS <- function(x, y, z = NULL, bw, est = c("lc", "ll"),
     theta[t,] <- result$coef[1:nvar]
     fitted[t] <- crossprod(x[t, !is.na(theta[t,])], theta[t, !is.na(theta[t,])])
   }
-  resid <- y - fitted
+  if(!is.predict)
+    resid <- y - fitted
   return(list(tvcoef = theta, fitted = fitted, residuals = resid))
 }
 
+#' @rdname tvOLS
+#' @method tvOLS tvlm
+#' @export
+tvOLS.tvlm <- function(x, ...)
+{
+  if(!inherits(x, c("tvlm", "tvar")))
+    stop ("Function for object of class 'tvlm' and 'tvar'. \n")
+  y <- x$y
+  y <- as.matrix(y)
+  tkernel <- x$tkernel
+  est <- x$est
+  obs <- x$obs
+  bw <- x$bw
+  z <- x$z
+  ez <- x$ez
+  singular.ok <- x$singular.ok
+  return(tvOLS(x = x$x, y, z, ez, bw, est, tkernel, singular.ok))
+}
+  
+#' @rdname tvOLS
+#' @method tvOLS tvar
+#' @export
+tvOLS.tvar <- tvOLS.tvlm
+
+#' @rdname tvOLS
+#' @method tvOLS tvvar
+#' @export
+tvOLS.tvvar <- function(x, ...)
+{
+  if(!inherits(x, c("tvvar")))
+    stop ("Function for object of class 'tvvar'. \n")
+  neq <- x$neq
+  yend <- x$y
+  rhs <- x$x
+  z <- x$z
+  ez <- x$ez
+  bw <- x$bw
+  est <- x$est
+  tkernel <- x$tkernel
+  singular.ok <- x$singular.ok
+  equation <- list()
+  resid = fitted <- matrix(0, nrow = x$obs, ncol = neq)
+  for (i in 1:neq)
+  {
+    y <- yend[, i]
+    results <- tvOLS(x = rhs, y = y, z = z, ez = ez, bw = bw[i], 
+                     est = est, tkernel = tkernel, singular.ok = singular.ok)
+    equation[[colnames(yend)[i]]] <- results$tvcoef
+    colnames(equation[[colnames(yend)[i]]]) <- colnames(rhs)
+    resid[,i] <- results$residuals
+    fitted[, i] <- results$fitted
+  }
+  
+  return(list(tvcoef = equation, fitted = fitted, residuals = resid))
+}
+
 
 #' @rdname tvReg-internals
 #' @keywords internal
